/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */


/*
 * Logger.cpp
 *
 *  Created on: 2010/12/13
 *      Author: HARK Development Team
 */

#include <Logger.hpp>
#include <iostream>
#include <ctime>
using namespace std;
using namespace boost;

namespace NSHarkBinaural
{

const int LogLevel::ALL = std::numeric_limits<int>::min();
const int LogLevel::OFF = std::numeric_limits<int>::max();
const string Logger::GLOBAL_LOGGER_NAME = "global";
std::map<std::string, boost::shared_ptr<Logger> > Logger::loggers;

std::string LogLevel::ToString()
{
    if (this->level < FINEST) return "ALL";
    switch (this->level)
    {
    case FINEST:
        return "FINEST";
    case FINER:
        return "FINER";
    case FINE:
        return "FINE";
    case CONFIG:
        return "CONFIG";
    case INFO:
        return "INFO";
    case WARNING:
        return "WARNING";
    case SEVERE:
        return "SEVERE";
    default:
        return "";
    }
}

Logger& Logger::GetLogger(const std::string& name)
{
    if (Logger::loggers.find(name) != Logger::loggers.end())
    {
        return *Logger::loggers[name];
    }
    else if (name == Logger::GLOBAL_LOGGER_NAME)
    {
        CreateLogger(name);
        shared_ptr<Logger> global = Logger::loggers[name];
        global->AddHandler(&cout);
        return *global;
    }
    else
    {
        CreateLogger(name);
        return *Logger::loggers[name];
    }
}

void Logger::AddHandler(ostream *os, bool needDelete)
{
    for (std::vector<OstreamPtr>::iterator it = this->ostreams.begin(); it != this->ostreams.end(); ++it)
    {
        if (it->ptr == os) return;
    }
    OstreamPtr p;
    p.ptr = os;
    p.needDelete = needDelete;
    this->ostreams.push_back(p);
}

void Logger::Log(LogLevel level, const std::string& msg, const std::string& file, const std::string& func)
{
    if (level.level >= this->logLevel.level)
    {
        time_t current;
        tm *local;

        time(&current);
        local = localtime(&current);
        for (std::vector<OstreamPtr>::iterator it = this->ostreams.begin(); it != this->ostreams.end(); ++it)
        {
            *it->ptr << 1900 + local->tm_year << '/' << 1 + local->tm_mon << '/' << local->tm_mday << ' '
                     << local->tm_hour << ':' << local->tm_min << ':' << local->tm_sec << ' '
                     << file << ' ' << func << '\n'
                     << level.ToString() << ": " << msg << endl;
        }
    }
}

Logger::Logger(const std::string& name)
    : logLevel(LogLevel::INFO), name(name), ostreams()
{
}

Logger::~Logger()
{
    for (std::vector<OstreamPtr>::iterator it = this->ostreams.begin(); it != this->ostreams.end(); ++it)
    {
        if (it->needDelete)
        {
            delete it->ptr;
            it->ptr = 0;
        }
    }
}

void Logger::CreateLogger(const std::string& name)
{
    shared_ptr<Logger> logger(new Logger(name));
    Logger::loggers.insert( map<string, shared_ptr<Logger> >::value_type( name, logger ) );
}


}
